#!/bin/bash

main()
{

#Loads needed: intel/2019.4 gcc impi/2017.4 netcdf/4.4.1.1 hdf5/1.8.19 EXTRAE/3.7.1 perl Paraver Dinemas


if [ $# -gt 0 ]; then

  echo "This script does not accept arguments, parameters need to be added to config.yaml Aborting"
  exit 1

fi

echo
echo "Using the following configuration:"
echo

Parse_yaml config.yaml
echo

yaml_string="$(Parse_yaml config.yaml)"
eval "${yaml_string}"

Init
Test_arguments
Test_Comp
Create_metrics

}

#yaml Parser
Parse_yaml()
 {
   local prefix=$2
   local s='[[:space:]]*' w='[a-zA-Z0-9_]*' fs=$(echo @|tr @ '\034')
   sed -ne "s|^\($s\):|\1|" \
        -e "s|^\($s\)\($w\)$s:$s[\"']\(.*\)[\"']$s\$|\1$fs\2$fs\3|p" \
        -e "s|^\($s\)\($w\)$s:$s\(.*\)$s\$|\1$fs\2$fs\3|p"  $1 |
   awk -F$fs '{
      indent = length($1)/2;
      vname[indent] = $2;
      for (i in vname) {if (i > indent) {delete vname[i]}}
      if (length($3) > 0) {
         vn=""; for (i=0; i<indent; i++) {vn=(vn)(vname[i])("_")}
         printf("%s%s%s=\"%s\"\n", "'$prefix'",vn, $2, $3);
      }
   }'
}

Init()
{

  #Init variables with default values in case of missing
  dir=$(pwd)
  Nemo_path="${Nemo_path:-"."}"
  Jobs_n_cores="${Jobs_n_cores:-48}"
  Jobs_scheduler="${Jobs_scheduler:-"slurm"}"
  time="${Jobs_time:-"0"}"
  queue="${Jobs_queue:-""}"
  compile="${Compilation_compile:-"false"}"
  cfg="${Compilation_ref:-"ORCA2_ICE_PISCES"}"
  arch="${Compilation_arch:-"X64_MN4"}"
  name_cfg="${Compilation_name:-"ORCA2_EXTRAE"}"
  comp_cfg="${Compilation_sub:-""}"
  Modules="${Modules:-""}"

  module load $Modules
}


# Checks if the paths given are correct.

Test_arguments()
{
  # Nemo path correct?
  if ! test -d "${Nemo_path}"; then
    echo "Nemo relative path: ${Nemo_path} is not found"
    echo
    exit 1
  fi

  # cfg exists?
  if ! test -d "${Nemo_path}/cfgs/${cfg}"; then
    echo "configuration: ${cfg} doesn't exists in ${Nemo_path}/cfgs dir"
    echo
    exit 1
  fi
  

   # arch exists?
  if ! test -f "${Nemo_path}/arch/arch-${arch}.fcm"; then
    echo "architecture: arch-${arch}.fcm doesn't exists in ${Nemo_path}/arch dir"
    echo
    exit 1
  fi

  # scheduler correct?
  if [ $Jobs_scheduler != "slurm" ] && [ $Jobs_scheduler != "lsf" ]; then
    echo "$Jobs_scheduler is not a valid scheduler"
    echo
    exit 1
  fi

  #$EXTRAE_HOME correct ?
  if ! test -d "${EXTRAE_HOME}"; then
  echo "Extrae relative path: ${EXTRAE_HOME} is not found"
  echo
 # exit 1
  else
  sed -i 's|home=.*|home="'"$EXTRAE_HOME"'"|g' extrae.xml
  fi

  # Adding -d to variable if not empty
  if [ ! -z "$comp_cfg" ]; then
    comp_cfg="-d $comp_cfg"
  fi
  
  # Creating auxiliar vars for submiting jobs
  if [ $Jobs_scheduler == "slurm" ]; then 
    job="sbatch"
  else
    job="bsub"
  fi

  
}


# Checks if compiler flag -g is present in the arch file, this flag is needed for the correct creation of traces.



Test_Comp()
{

  # Checking if compilation is needed

  if [ $compile == true ]; then
      echo '"$compile" parameter is inicialized true '
  fi
  
  # Get text line corresponding to the flags
  line=$(sed -n '/^%FCFLAGS /p' $Nemo_path/arch/arch-${arch}.fcm)

  # If -g is not there, recompilation is requiered and -g added
  if ! echo "${line}"|grep -q "\-g\b"; then
    echo "-g flag not found in arch-${arch}.fcm: editing arch-${arch}.fcm "
    sed -i '/^%FCFLAGS/ s/$/ -g /' ${Nemo_path}/arch/arch-${arch}.fcm
    compile=true
  fi

  # If finstrument-functions is not there recompilation is requiered and -finstrument-functions added
  if ! echo "${line}"|grep -q "\-finstrument-functions\b"; then
    echo "-finstrument-functions flag not found in arch-${arch}.fcm: editing arch-${arch}.fcm "
    sed -i '/^%FCFLAGS/ s/$/ -finstrument-functions/' ${Nemo_path}/arch/arch-${arch}.fcm
    compile=true
  fi

  # If -pg is not there recompilation is requiered and -pg added
  
  line2=$(sed -n '/^%FPPFLAGS /p' $Nemo_path/arch/arch-${arch}.fcm)
  line3=$(sed -n '/^%LDFLAGS /p' $Nemo_path/arch/arch-${arch}.fcm)
   

  if ! echo "${line}"|grep -q "\-pg\b"; then
    echo "-pg flag not found in FCFLAGS arch-${arch}.fcm: editing arch-${arch}.fcm "
    sed -i '/^%FCFLAGS/ s/$/ -pg/' ${Nemo_path}/arch/arch-${arch}.fcm
    compile=true
  fi

  if ! echo "${line2}"|grep -q "\-pg\b"; then
    echo "-pg flag not found in FPPFLAGS arch-${arch}.fcm : editing arch-${arch}.fcm "
    sed -i '/^%FPPFLAGS/ s/$/ -pg/' ${Nemo_path}/arch/arch-${arch}.fcm
    compile=true
  fi

  if ! echo "${line3}"|grep -q "\-pg\b"; then
    echo "-pg flag not found in LDFLAGS arch-${arch}.fcm: editing arch-${arch}.fcm "
    sed -i '/^%LDFLAGS/ s/$/ -pg/' ${Nemo_path}/arch/arch-${arch}.fcm
    compile=true
  fi


  # If -rdynamic is not there recompilation is requiered and -rdynamic added
  
  if ! echo "${line}"|grep -q "\-rdynamic\b"; then
    echo "-rdynamic flag not found in FCFLAGS arch-${arch}.fcm: editing arch-${arch}.fcm "
    sed -i '/^%FCFLAGS/ s/$/ -rdynamic/' ${Nemo_path}/arch/arch-${arch}.fcm
    compile=true
  fi

 if ! echo "${line3}"|grep -q "\-export-dynamic\b"; then
    echo "-export-dynamic flag not found in LDFLAGS arch-${arch}.fcm: editing arch-${arch}.fcm "
    sed -i '/^%LDFLAGS/ s/$/ -export-dynamic/' ${Nemo_path}/arch/arch-${arch}.fcm
    compile=true
  fi
  
  

  #Compile the program if needed

  if [ $compile == true ]; then
    echo "Compiling Nemo, expected duration 35m"
    printf -v workload1 "cd ${Nemo_path}\n./makenemo -r ${cfg} -n ${name_cfg} -m ${arch} -j$Jobs_n_cores $comp_cfg"
    python3 Job_Creator.py -f "compile" -j "compile" --set-core ${Jobs_n_cores} -s $Jobs_scheduler --set-time $time --set-queue $queue -w "${workload1}"
    $job --wait compile.$Jobs_scheduler

    #Rename the namelist_cfg if exists in order to not overwrite it
    if test -f "namelist_cfg"; then
      mv namelist_cfg namelist_cfg_old
      cd $dir
      cp ${Nemo_path}/cfgs/${name_cfg}/EXP00/* .
      rm namelist_cfg
      mv namelist_cfg_old namelist_cfg
    else
      cd $dir
      cp ${Nemo_path}/cfgs/${name_cfg}/EXP00/* .
    fi

    if [[ $comp_cfg == "-d OCE del_key 'key_si3 key_top'" ]]; then
      sed -i '/_def_nemo-ice.xml\|def_nemo-pisces.xml/d' context_nemo.xml #DELETE ICE AND PISCES CONTEXT (NOT USED)
    fi

    #Solving NEMO 4.2 Errors
    sed -i 's|ln_zdfiwm * = .true.|ln_zdfiwm   = .false.|g' namelist_cfg #CHANGE DUE TO NON EXISTING FILES
    if test -f "weights_core_orca2_bicubic_noc.nc"; then
      mv weights_core_orca2_bicubic_noc.nc weights_core2_orca2_bicub.nc #RENAME WRONG NAMED FILES
    fi
    if test -f "weights_core_orca2_bilinear_noc.nc"; then 
      mv weights_core_orca2_bilinear_noc.nc weights_core2_orca2_bilin.nc #RENAME WRONG NAMED FILES
    fi
  else

    echo "Compilation not needed"  
  
  fi

}

Create_metrics()
{


  #Changing iterations, big traces generate problems.
  sed -i 's|nn_itend * =.*|nn_itend    =       12  !  last  time step (std 5475)|g' namelist_cfg

  #Generating function list in case of missing

  if ! test -f "extrae_functions_for_xml.txt"; then

    echo "Runing Nemo with 4 cores to obtain function data..."
    echo
    python3 Job_Creator.py -f "run" -j "run" --set-core 4 -s $Jobs_scheduler --set-time $time --set-queue $queue -w "mpirun -np 4 ./nemo" 
    $job --wait run.$Jobs_scheduler
    echo " Gthrottling functions ..."
    ./gthrottling.sh nemo
    ./extraf.sh  nemo extrae_functions.txt
  
  else
    echo "Functions already listed file nemo, extrae_functions.txt does exist"
    echo
  fi

  sed -i "s|list=.*|list=\"${dir}/extrae_functions_for_xml.txt\" exclude-automatic-functions=\"yes\">|g" extrae.xml
  
  # Run nemo with extrae
  echo "Creating trace ..."
  echo
  python3 Job_Creator.py -f "run_extrae" -j "run_extrae" --set-core ${Jobs_n_cores} -s $Jobs_scheduler --set-time $time --set-queue $queue -w "mpirun -np ${Jobs_n_cores} ./trace.sh ./nemo" 
  $job --wait run_extrae.$Jobs_scheduler

  echo "Cutting best iteration"
  echo
  magiccut/./magicCut nemo.prv 12



  echo "Creating metrics and storing theme in Metrics_${Jobs_n_cores}_cores folder"
  echo
  # Creating folder
  if ! test -d "Metrics${Jobs_n_cores}cores"; then
    mkdir Metrics_${Jobs_n_cores}_cores
  fi
 
  cp nemo.best_cut.* Metrics_${Jobs_n_cores}_cores

  python3 Job_Creator.py -f "modelfactors" -j "modelfactors" --set-core ${Jobs_n_cores} -s $Jobs_scheduler --set-time $time --set-queue $queue -w "modelfactors.py nemo.best_cut.prv" 
  mv modelfactors.$Jobs_scheduler Metrics_${Jobs_n_cores}_cores
  cd Metrics_${Jobs_n_cores}_cores
  $job --wait modelfactors.$Jobs_scheduler
  
}
main "$@"; exit
